/*
 * Copyright (C) 2016 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/

#ifndef GAZEBO_PLUGINS_ACTORPLUGIN_HH_
#define GAZEBO_PLUGINS_ACTORPLUGIN_HH_

#include <string>
#include <vector>

#include "ros/ros.h"
#include <geometry_msgs/PoseStamped.h>
#include <geometry_msgs/TwistStamped.h>
#include <nav_msgs/Odometry.h>
#include <ctime>
#include <cstdio>
#include "gazebo/common/Plugin.hh"
#include "gazebo/physics/physics.hh"
#include "gazebo/util/system.hh"



namespace gazebo
{
  class GAZEBO_VISIBLE ActorPlugin : public ModelPlugin
  {
    /// \brief Constructor
    public: ActorPlugin();

    /// \brief Load the actor plugin.
    /// \param[in] _model Pointer to the parent model.
    /// \param[in] _sdf Pointer to the plugin's SDF elements.
    public: virtual void Load(physics::ModelPtr _model, sdf::ElementPtr _sdf);

    // Documentation Inherited.
    public: virtual void Reset();

    /// \brief Function that is called every update cycle.
    /// \param[in] _info Timing information
    private: void OnUpdate(const common::UpdateInfo &_info);

    /// \brief Helper function to choose a new target location
    private: void ChooseNewTarget();

    /// \brief Helper function to avoid obstacles. This implements a very
    /// simple vector-field algorithm.
    /// \param[in] _pos Direction vector that should be adjusted according
    /// to nearby obstacles.
    private: void HandleObstacles(ignition::math::Vector3d &_pos);

    private: double heightMapZ(double, double);

    /// \brief Pointer to the parent actor.
    private: physics::ActorPtr actor;

    /// \brief Pointer to the world, for convenience.
    private: physics::WorldPtr world;

    /// \brief Pointer to the sdf element.
    private: sdf::ElementPtr sdf;

    /// \brief Variable to indicate simtime start.
    private: double start;

    /// \brief Pause duration for animation.
    private: double duration;

    /// \brief Velocity of the actor
    private: ignition::math::Vector3d velocity;

    /// \brief List of connections
    private: std::vector<event::ConnectionPtr> connections;

    /// \brief Current target location
    private: ignition::math::Vector3d target;

    /// \brief Target location weight (used for vector field)
    private: double targetWeight = 1.0;

    /// \brief Obstacle weight (used for vector field)
    private: double obstacleWeight = 1.0;

    /// \brief Time scaling factor. Used to coordinate translational motion
    /// with the actor's walking animation.
    private: double animationFactor = 1.0;

    /// \brief Time of the last update.
    private: common::Time lastUpdate;

    /// \brief List of models to ignore. Used for vector field
    private: std::vector<std::string> ignoreModels;

    /// \brief Custom trajectory info.
    private: physics::TrajectoryInfoPtr trajectoryInfo;

    // use heightmap if terrain is defined
    private: int USE_HEIGHT_MAP=0;

    //define person_walking_height
    private: int walking_height=1.0;


  };
//  Custom Trajectory
   std::vector<double> x_waypoints = { 0,0.025042,0.050083,0.075125,0.10017,0.12521,0.15025,0.17529,0.20033,0.22538,0.25042,0.27546,0.3005,0.32554,0.35058,0.37563,0.40067,0.42571,0.45075,0.47579,0.50083,0.52588,0.55092,0.57596,0.601,0.62604,0.65109,0.67613,0.70117,0.72621,0.75125,0.77629,0.80134,0.82638,0.85142,0.87646,0.9015,0.92654,0.95159,0.97663,1.0017,1.0267,1.0518,1.0768,1.1018,1.1269,1.1519,1.177,1.202,1.227,1.2521,1.2771,1.3022,1.3272,1.3523,1.3773,1.4023,1.4274,1.4524,1.4775,1.5025,1.5275,1.5526,1.5776,1.6027,1.6277,1.6528,1.6778,1.7028,1.7279,1.7529,1.778,1.803,1.828,1.8531,1.8781,1.9032,1.9282,1.9533,1.9783,2.0033,2.0284,2.0534,2.0785,2.1035,2.1285,2.1536,2.1786,2.2037,2.2287,2.2538,2.2788,2.3038,2.3289,2.3539,2.379,2.404,2.429,2.4541,2.4791,2.5042,2.5292,2.5543,2.5793,2.6043,2.6294,2.6544,2.6795,2.7045,2.7295,2.7546,2.7796,2.8047,2.8297,2.8548,2.8798,2.9048,2.9299,2.9549,2.98,3.005,3.0301,3.0551,3.0801,3.1052,3.1302,3.1553,3.1803,3.2053,3.2304,3.2554,3.2805,3.3055,3.3306,3.3556,3.3806,3.4057,3.4307,3.4558,3.4808,3.5058,3.5309,3.5559,3.581,3.606,3.6311,3.6561,3.6811,3.7062,3.7312,3.7563,3.7813,3.8063,3.8314,3.8564,3.8815,3.9065,3.9316,3.9566,3.9816,4.0067,4.0317,4.0568,4.0818,4.1068,4.1319,4.1569,4.182,4.207,4.2321,4.2571,4.2821,4.3072,4.3322,4.3573,4.3823,4.4073,4.4324,4.4574,4.4825,4.5075,4.5326,4.5576,4.5826,4.6077,4.6327,4.6578,4.6828,4.7078,4.7329,4.7579,4.783,4.808,4.8331,4.8581,4.8831,4.9082,4.9332,4.9583,4.9833,5.0083,5.0334,5.0584,5.0835,5.1085,5.1336,5.1586,5.1836,5.2087,5.2337,5.2588,5.2838,5.3088,5.3339,5.3589,5.384,5.409,5.4341,5.4591,5.4841,5.5092,5.5342,5.5593,5.5843,5.6093,5.6344,5.6594,5.6845,5.7095,5.7346,5.7596,5.7846,5.8097,5.8347,5.8598,5.8848,5.9098,5.9349,5.9599,5.985,6.01,6.0351,6.0601,6.0851,6.1102,6.1352,6.1603,6.1853,6.2104,6.2354,6.2604,6.2855,6.3105,6.3356,6.3606,6.3856,6.4107,6.4357,6.4608,6.4858,6.5109,6.5359,6.5609,6.586,6.611,6.6361,6.6611,6.6861,6.7112,6.7362,6.7613,6.7863,6.8114,6.8364,6.8614,6.8865,6.9115,6.9366,6.9616,6.9866,7.0117,7.0367,7.0618,7.0868,7.1119,7.1369,7.1619,7.187,7.212,7.2371,7.2621,7.2871,7.3122,7.3372,7.3623,7.3873,7.4124,7.4374,7.4624,7.4875,7.5125,7.5376,7.5626,7.5876,7.6127,7.6377,7.6628,7.6878,7.7129,7.7379,7.7629,7.788,7.813,7.8381,7.8631,7.8881,7.9132,7.9382,7.9633,7.9883,8.0134,8.0384,8.0634,8.0885,8.1135,8.1386,8.1636,8.1886,8.2137,8.2387,8.2638,8.2888,8.3139,8.3389,8.3639,8.389,8.414,8.4391,8.4641,8.4891,8.5142,8.5392,8.5643,8.5893,8.6144,8.6394,8.6644,8.6895,8.7145,8.7396,8.7646,8.7896,8.8147,8.8397,8.8648,8.8898,8.9149,8.9399,8.9649,8.99,9.015,9.0401,9.0651,9.0902,9.1152,9.1402,9.1653,9.1903,9.2154,9.2404,9.2654,9.2905,9.3155,9.3406,9.3656,9.3907,9.4157,9.4407,9.4658,9.4908,9.5159,9.5409,9.5659,9.591,9.616,9.6411,9.6661,9.6912,9.7162,9.7412,9.7663,9.7913,9.8164,9.8414,9.8664,9.8915,9.9165,9.9416,9.9666,9.9917,10.017,10.042,10.067,10.092,10.117,10.142,10.167,10.192,10.217,10.242,10.267,10.292,10.317,10.342,10.367,10.392,10.417,10.442,10.467,10.492,10.518,10.543,10.568,10.593,10.618,10.643,10.668,10.693,10.718,10.743,10.768,10.793,10.818,10.843,10.868,10.893,10.918,10.943,10.968,10.993,11.018,11.043,11.068,11.093,11.119,11.144,11.169,11.194,11.219,11.244,11.269,11.294,11.319,11.344,11.369,11.394,11.419,11.444,11.469,11.494,11.519,11.544,11.569,11.594,11.619,11.644,11.669,11.694,11.72,11.745,11.77,11.795,11.82,11.845,11.87,11.895,11.92,11.945,11.97,11.995,12.02,12.045,12.07,12.095,12.12,12.145,12.17,12.195,12.22,12.245,12.27,12.295,12.321,12.346,12.371,12.396,12.421,12.446,12.471,12.496,12.521,12.546,12.571,12.596,12.621,12.646,12.671,12.696,12.721,12.746,12.771,12.796,12.821,12.846,12.871,12.896,12.922,12.947,12.972,12.997,13.022,13.047,13.072,13.097,13.122,13.147,13.172,13.197,13.222,13.247,13.272,13.297,13.322,13.347,13.372,13.397,13.422,13.447,13.472,13.497,13.523,13.548,13.573,13.598,13.623,13.648,13.673,13.698,13.723,13.748,13.773,13.798,13.823,13.848,13.873,13.898,13.923,13.948,13.973,13.998,14.023,14.048,14.073,14.098,14.124,14.149,14.174,14.199,14.224,14.249,14.274,14.299,14.324,14.349,14.374,14.399,14.424,14.449,14.474,14.499,14.524,14.549,14.574,14.599,14.624,14.649,14.674,14.699,14.725,14.75,14.775,14.8,14.825,14.85,14.875,14.9,14.925,14.95,14.975,15,15,14.999,14.998,14.997,14.995,14.993,14.99,14.987,14.983,14.979,14.975,14.97,14.965,14.959,14.953,14.947,14.94,14.933,14.925,14.917,14.909,14.9,14.891,14.881,14.871,14.86,14.849,14.838,14.826,14.814,14.802,14.789,14.775,14.761,14.747,14.733,14.718,14.702,14.686,14.67,14.653,14.636,14.619,14.601,14.583,14.564,14.545,14.526,14.506,14.485,14.465,14.444,14.422,14.4,14.378,14.356,14.332,14.309,14.285,14.261,14.236,14.211,14.186,14.16,14.134,14.107,14.08,14.053,14.025,13.997,13.969,13.94,13.91,13.881,13.851,13.82,13.789,13.758,13.727,13.695,13.662,13.63,13.596,13.563,13.529,13.495,13.46,13.425,13.39,13.354,13.318,13.282,13.245,13.208,13.17,13.132,13.094,13.056,13.017,12.977,12.938,12.897,12.857,12.816,12.775,12.734,12.692,12.65,12.607,12.564,12.521,12.478,12.434,12.389,12.345,12.3,12.255,12.209,12.163,12.117,12.07,12.023,11.976,11.928,11.88,11.832,11.783,11.734,11.685,11.636,11.586,11.536,11.485,11.434,11.383,11.332,11.28,11.228,11.175,11.123,11.07,11.016,10.963,10.909,10.854,10.8,10.745,10.69,10.634,10.579,10.523,10.466,10.41,10.353,10.296,10.238,10.181,10.123,10.064,10.006,9.9468,9.8877,9.8283,9.7687,9.7087,9.6485,9.5881,9.5273,9.4663,9.4051,9.3435,9.2818,9.2197,9.1574,9.0949,9.0321,8.9691,8.9058,8.8423,8.7785,8.7145,8.6502,8.5857,8.521,8.456,8.3908,8.3254,8.2597,8.1938,8.1277,8.0613,7.9948,7.928,7.861,7.7938,7.7263,7.6587,7.5908,7.5227,7.4545,7.386,7.3173,7.2484,7.1793,7.11,7.0405,6.9708,6.901,6.8309,6.7607,6.6902,6.6196,6.5488,6.4778,6.4066,6.3353,6.2638,6.1921,6.1202,6.0482,5.976,5.9037,5.8311,5.7584,5.6856,5.6126,5.5394,5.4661,5.3927,5.3191,5.2453,5.1714,5.0974,5.0232,4.9489,4.8744,4.7998,4.7251,4.6502,4.5753,4.5001,4.4249,4.3496,4.2741,4.1985,4.1228,4.047,3.971,3.895,3.8188,3.7426,3.6662,3.5897,3.5132,3.4365,3.3598,3.2829,3.206,3.129,3.0518,2.9746,2.8974,2.82,2.7426,2.6651,2.5875,2.5098,2.4321,2.3543,2.2764,2.1985,2.1205,2.0425,1.9644,1.8863,1.8081,1.7298,1.6515,1.5732,1.4948,1.4163,1.3379,1.2594,1.1808,1.1022,1.0236,0.945,0.86635,0.78766,0.70896,0.63023,0.55149,0.47274,0.39397,0.31519,0.2364,0.1576,0.078802,-2.7555e-15,3.2514,3.2522,3.2546,3.2585,3.264,3.2711,3.2797,3.2898,3.3015,3.3147,3.3293,3.3455,3.3632,3.3823,3.4028,3.4247,3.4479,3.4726,3.4985,3.5257,3.5542,3.5838,3.6147,3.6466,3.6796,3.7137,3.7487,3.7847,3.8215,3.8591,3.8975,3.9366,3.9763,4.0165,4.0572,4.0983,4.1397,4.1814,4.2231,4.265,4.3068,4.3485,4.3899,4.431,4.4716,4.5117,4.5511,4.5898,4.6275,4.6642,4.6997,4.734,4.7668,4.7981,4.8277,4.8555,4.8813,4.9049,4.9263,4.9453,4.9617,4.9754,4.9863,4.9941,4.9987,5,4.9978,4.9919,4.9823,4.9687,4.951,4.9291,4.9028,4.872,4.8365,4.7962,4.7509,4.7006,4.6452,4.5844,4.5183,4.4467,4.3696,4.2867,4.1982,4.1039,4.0038,3.8978,3.786,3.6683,3.5446,3.4152,3.2799,3.1388,2.992,2.8395,2.6816,2.5182,2.3495,2.1757,1.997,1.8135,1.6254,1.4331,1.2366,1.0364,0.83273,0.62586,0.41614,0.20394,-0.010362,-0.22638,-0.4437,-0.66188,-0.88048,-1.099,-1.317,-1.534,-1.7494,-1.9627,-2.1733,-2.3808,-2.5846,-2.784,-2.9785,-3.1676,-3.3506,-3.5269,-3.696,-3.8573,-4.0102,-4.1541,-4.2886,-4.4129,-4.5267,-4.6294,-4.7204,-4.7994,-4.8659,-4.9195,-4.9598,-4.9863,-4.9989,-4.9973,-4.9811,-4.9502,-4.9045,-4.8438,-4.7682,-4.6775,-4.5718,-4.4513,-4.3161,-4.1664,-4.0025,-3.8248,-3.6335,-3.4293,-3.2126,-2.9839,-2.7439,-2.4933,-2.2329,-1.9634,-1.6857,-1.4007,-1.1094,-0.81274,-0.51179,-0.20765,0.098579,0.40573,0.71262,1.018,1.3208,1.6196,1.9132,2.2004,2.48,2.7506,3.0112,3.2604,3.4971,3.7201,3.9284,4.121,4.2967,4.4547,4.5941,4.7141,4.8139,4.8929,4.9505,4.9863,4.9999,4.9909,4.9593,4.9049,4.8278,4.7282,4.6062,4.4623,4.297,4.1108,3.9045,3.6789,3.435,3.1737,2.8962,2.6037,2.2977,1.9795,1.6505,1.3125,0.967,0.61574,0.26049,-0.096971,-0.45479,-0.81112,-1.1641,-1.5118,-1.8524,-2.1841,-2.5049,-2.8132,-3.1072,-3.3852,-3.6456,-3.8869,-4.1077,-4.3066,-4.4824,-4.634,-4.7604,-4.8608,-4.9345,-4.9809,-4.9996,-4.9903,-4.9531,-4.8878,-4.7948,-4.6745,-4.5274,-4.3542,-4.1558,-3.9332,-3.6877,-3.4205,-3.1332,-2.8272,-2.5044,-2.1665,-1.8154,-1.4533,-1.0822,-0.70426,-0.3217,0.063202,0.44816,0.83087,1.209,1.5804,1.9426,2.2935,2.631,2.9529,3.2573,3.5422,3.806,4.0468,4.2633,4.454,4.6177,4.7533,4.8599,4.9369,4.9837,5,4.9856,4.9405,4.865,4.7595,4.6246,4.4611,4.27,4.0525,3.8098,3.5435,3.2552,2.9467,2.6198,2.2766,1.9193,1.55,1.171,0.78475,0.3936,-1.3777e-14,0
};

std::vector<double> y_waypoints =  {
0,0.37559,0.75094,1.1258,1.5,1.8732,2.2453,2.6159,2.9849,3.3521,3.7171,4.0798,4.44,4.7973,5.1517,5.5028,5.8505,6.1945,6.5346,6.8707,7.2024,7.5296,7.852,8.1696,8.482,8.7891,9.0907,9.3867,9.6767,9.9606,10.238,10.51,10.774,11.032,11.283,11.527,11.764,11.993,12.215,12.429,12.636,12.834,13.024,13.207,13.381,13.546,13.703,13.852,13.991,14.122,14.245,14.358,14.462,14.557,14.643,14.72,14.788,14.846,14.895,14.935,14.965,14.986,14.998,15,14.992,14.976,14.95,14.914,14.869,14.815,14.752,14.679,14.597,14.506,14.406,14.297,14.179,14.052,13.916,13.772,13.619,13.457,13.287,13.108,12.922,12.727,12.524,12.313,12.095,11.869,11.636,11.395,11.147,10.892,10.631,10.362,10.088,9.8064,9.5191,9.2259,8.9268,8.6222,8.3122,7.9969,7.6767,7.3516,7.0219,6.6878,6.3495,6.0072,5.6612,5.3116,4.9587,4.6027,4.2438,3.8822,3.5182,3.152,2.7838,2.4138,2.0424,1.6697,1.2959,0.92131,0.54614,0.17063,-0.20499,-0.58047,-0.9556,-1.3301,-1.7038,-2.0764,-2.4478,-2.8175,-3.1856,-3.5516,-3.9154,-4.2767,-4.6354,-4.9911,-5.3437,-5.693,-6.0387,-6.3806,-6.7185,-7.0522,-7.3815,-7.7062,-8.026,-8.3408,-8.6503,-8.9544,-9.253,-9.5457,-9.8324,-10.113,-10.387,-10.655,-10.916,-11.17,-11.417,-11.657,-11.89,-12.115,-12.333,-12.543,-12.745,-12.939,-13.125,-13.303,-13.472,-13.633,-13.785,-13.929,-14.064,-14.19,-14.308,-14.416,-14.515,-14.605,-14.686,-14.758,-14.821,-14.874,-14.918,-14.952,-14.978,-14.993,-15,-14.997,-14.984,-14.963,-14.931,-14.891,-14.841,-14.782,-14.713,-14.636,-14.549,-14.453,-14.348,-14.234,-14.111,-13.979,-13.838,-13.689,-13.531,-13.365,-13.19,-13.007,-12.816,-12.617,-12.41,-12.195,-11.973,-11.743,-11.505,-11.261,-11.009,-10.75,-10.485,-10.213,-9.9349,-9.6504,-9.3598,-9.0634,-8.7613,-8.4537,-8.1408,-7.8228,-7.4998,-7.1722,-6.8401,-6.5037,-6.1632,-5.8188,-5.4708,-5.1194,-4.7648,-4.4072,-4.0468,-3.6838,-3.3186,-2.9513,-2.5821,-2.2113,-1.8391,-1.4658,-1.0916,-0.71662,-0.34124,0.034357,0.40993,0.78525,1.1601,1.5342,1.9073,2.2793,2.6498,3.0186,3.3856,3.7504,4.1129,4.4728,4.8299,5.184,5.5348,5.8821,6.2258,6.5655,6.9012,7.2325,7.5593,7.8813,8.1984,8.5103,8.817,9.1181,9.4134,9.7029,9.9863,10.263,10.534,10.798,11.055,11.306,11.549,11.785,12.014,12.235,12.448,12.654,12.852,13.041,13.223,13.396,13.561,13.717,13.865,14.004,14.134,14.255,14.368,14.471,14.565,14.651,14.727,14.793,14.851,14.899,14.938,14.967,14.987,14.998,14.999,14.991,14.974,14.947,14.911,14.865,14.81,14.746,14.672,14.589,14.498,14.397,14.287,14.168,14.04,13.903,13.758,13.604,13.442,13.271,13.091,12.904,12.708,12.505,12.294,12.075,11.848,11.614,11.373,11.124,10.869,10.606,10.337,10.062,9.7804,9.4926,9.1988,8.8992,8.5941,8.2836,7.9678,7.6471,7.3216,6.9915,6.657,6.3184,5.9757,5.6294,5.2795,4.9263,4.57,4.2108,3.849,3.4848,3.1184,2.75,2.3799,2.0084,1.6355,1.2617,0.88701,0.5118,0.13628,-0.23934,-0.6148,-0.98988,-1.3643,-1.7379,-2.1105,-2.4816,-2.8513,-3.2191,-3.5849,-3.9485,-4.3096,-4.668,-5.0235,-5.3758,-5.7248,-6.0701,-6.4117,-6.7492,-7.0825,-7.4114,-7.7356,-8.055,-8.3693,-8.6784,-8.982,-9.28,-9.5721,-9.8583,-10.138,-10.412,-10.679,-10.939,-11.193,-11.44,-11.679,-11.911,-12.135,-12.352,-12.562,-12.763,-12.956,-13.141,-13.318,-13.487,-13.647,-13.799,-13.942,-14.076,-14.201,-14.318,-14.425,-14.524,-14.613,-14.693,-14.764,-14.826,-14.878,-14.921,-14.955,-14.979,-14.994,-15,-14.996,-14.983,-14.96,-14.928,-14.887,-14.836,-14.776,-14.707,-14.628,-14.54,-14.444,-14.338,-14.223,-14.099,-13.967,-13.825,-13.675,-13.516,-13.349,-13.174,-12.99,-12.798,-12.598,-12.391,-12.175,-11.952,-11.721,-11.483,-11.238,-10.986,-10.726,-10.46,-10.188,-9.9091,-9.6241,-9.333,-9.036,-8.7334,-8.4253,-8.1119,-7.7934,-7.4701,-7.142,-6.8095,-6.4727,-6.1319,-5.7872,-5.4388,-5.0871,-4.7322,-4.3743,-4.0137,-3.6505,-3.2851,-2.9176,-2.5482,-2.1773,-1.805,-1.4316,-1.0573,-0.6823,-0.30689,0.068714,0.44428,0.81956,1.1943,1.5683,1.9414,2.3132,2.6836,3.0523,3.419,3.7837,4.1459,4.5056,4.8624,5.2162,5.5667,5.9137,6.257,6.5964,6.9317,7.2626,7.5889,7.9105,8.2271,8.5386,8.8447,9.1453,9.4401,9.7291,10.012,10.288,10.559,10.822,11.079,11.328,11.571,11.806,12.034,12.255,12.468,12.672,12.869,13.058,13.239,13.411,13.576,13.731,13.878,14.016,14.145,14.266,14.378,14.48,14.574,14.658,14.733,14.799,14.856,14.903,14.941,14.97,14.989,14.999,14.999,14.99,14.972,14.944,14.907,14.86,14.804,14.739,14.665,14.581,14.489,14.387,14.276,14.156,14.028,13.89,13.744,13.59,13.426,13.255,13.075,12.886,12.69,12.486,12.274,12.054,11.827,11.592,11.35,11.101,10.845,10.582,10.313,10.037,-3.6739e-15,-0.078802,-0.1576,-0.2364,-0.31519,-0.39397,-0.47274,-0.55149,-0.63023,-0.70896,-0.78766,-0.86635,-0.945,-1.0236,-1.1022,-1.1808,-1.2594,-1.3379,-1.4163,-1.4948,-1.5732,-1.6515,-1.7298,-1.8081,-1.8863,-1.9644,-2.0425,-2.1205,-2.1985,-2.2764,-2.3543,-2.4321,-2.5098,-2.5875,-2.6651,-2.7426,-2.82,-2.8974,-2.9746,-3.0518,-3.129,-3.206,-3.2829,-3.3598,-3.4365,-3.5132,-3.5897,-3.6662,-3.7426,-3.8188,-3.895,-3.971,-4.047,-4.1228,-4.1985,-4.2741,-4.3496,-4.4249,-4.5001,-4.5753,-4.6502,-4.7251,-4.7998,-4.8744,-4.9489,-5.0232,-5.0974,-5.1714,-5.2453,-5.3191,-5.3927,-5.4661,-5.5394,-5.6126,-5.6856,-5.7584,-5.8311,-5.9037,-5.976,-6.0482,-6.1202,-6.1921,-6.2638,-6.3353,-6.4066,-6.4778,-6.5488,-6.6196,-6.6902,-6.7607,-6.8309,-6.901,-6.9708,-7.0405,-7.11,-7.1793,-7.2484,-7.3173,-7.386,-7.4545,-7.5227,-7.5908,-7.6587,-7.7263,-7.7938,-7.861,-7.928,-7.9948,-8.0613,-8.1277,-8.1938,-8.2597,-8.3254,-8.3908,-8.456,-8.521,-8.5857,-8.6502,-8.7145,-8.7785,-8.8423,-8.9058,-8.9691,-9.0321,-9.0949,-9.1574,-9.2197,-9.2818,-9.3435,-9.4051,-9.4663,-9.5273,-9.5881,-9.6485,-9.7087,-9.7687,-9.8283,-9.8877,-9.9468,-10.006,-10.064,-10.123,-10.181,-10.238,-10.296,-10.353,-10.41,-10.466,-10.523,-10.579,-10.634,-10.69,-10.745,-10.8,-10.854,-10.909,-10.963,-11.016,-11.07,-11.123,-11.175,-11.228,-11.28,-11.332,-11.383,-11.434,-11.485,-11.536,-11.586,-11.636,-11.685,-11.734,-11.783,-11.832,-11.88,-11.928,-11.976,-12.023,-12.07,-12.117,-12.163,-12.209,-12.255,-12.3,-12.345,-12.389,-12.434,-12.478,-12.521,-12.564,-12.607,-12.65,-12.692,-12.734,-12.775,-12.816,-12.857,-12.897,-12.938,-12.977,-13.017,-13.056,-13.094,-13.132,-13.17,-13.208,-13.245,-13.282,-13.318,-13.354,-13.39,-13.425,-13.46,-13.495,-13.529,-13.563,-13.596,-13.63,-13.662,-13.695,-13.727,-13.758,-13.789,-13.82,-13.851,-13.881,-13.91,-13.94,-13.969,-13.997,-14.025,-14.053,-14.08,-14.107,-14.134,-14.16,-14.186,-14.211,-14.236,-14.261,-14.285,-14.309,-14.332,-14.356,-14.378,-14.4,-14.422,-14.444,-14.465,-14.485,-14.506,-14.526,-14.545,-14.564,-14.583,-14.601,-14.619,-14.636,-14.653,-14.67,-14.686,-14.702,-14.718,-14.733,-14.747,-14.761,-14.775,-14.789,-14.802,-14.814,-14.826,-14.838,-14.849,-14.86,-14.871,-14.881,-14.891,-14.9,-14.909,-14.917,-14.925,-14.933,-14.94,-14.947,-14.953,-14.959,-14.965,-14.97,-14.975,-14.979,-14.983,-14.987,-14.99,-14.993,-14.995,-14.997,-14.998,-14.999,-15,-15,-15,-14.95,-14.9,-14.849,-14.799,-14.749,-14.699,-14.649,-14.599,-14.548,-14.498,-14.448,-14.398,-14.348,-14.298,-14.247,-14.197,-14.147,-14.097,-14.047,-13.997,-13.946,-13.896,-13.846,-13.796,-13.746,-13.696,-13.645,-13.595,-13.545,-13.495,-13.445,-13.395,-13.344,-13.294,-13.244,-13.194,-13.144,-13.094,-13.043,-12.993,-12.943,-12.893,-12.843,-12.793,-12.742,-12.692,-12.642,-12.592,-12.542,-12.492,-12.441,-12.391,-12.341,-12.291,-12.241,-12.191,-12.14,-12.09,-12.04,-11.99,-11.94,-11.89,-11.839,-11.789,-11.739,-11.689,-11.639,-11.589,-11.538,-11.488,-11.438,-11.388,-11.338,-11.288,-11.237,-11.187,-11.137,-11.087,-11.037,-10.987,-10.936,-10.886,-10.836,-10.786,-10.736,-10.686,-10.635,-10.585,-10.535,-10.485,-10.435,-10.385,-10.334,-10.284,-10.234,-10.184,-10.134,-10.084,-10.033,-9.9833,-9.9331,-9.8829,-9.8328,-9.7826,-9.7324,-9.6823,-9.6321,-9.5819,-9.5318,-9.4816,-9.4314,-9.3813,-9.3311,-9.2809,-9.2308,-9.1806,-9.1304,-9.0803,-9.0301,-8.9799,-8.9298,-8.8796,-8.8294,-8.7793,-8.7291,-8.6789,-8.6288,-8.5786,-8.5284,-8.4783,-8.4281,-8.3779,-8.3278,-8.2776,-8.2274,-8.1773,-8.1271,-8.0769,-8.0268,-7.9766,-7.9264,-7.8763,-7.8261,-7.7759,-7.7258,-7.6756,-7.6254,-7.5753,-7.5251,-7.4749,-7.4247,-7.3746,-7.3244,-7.2742,-7.2241,-7.1739,-7.1237,-7.0736,-7.0234,-6.9732,-6.9231,-6.8729,-6.8227,-6.7726,-6.7224,-6.6722,-6.6221,-6.5719,-6.5217,-6.4716,-6.4214,-6.3712,-6.3211,-6.2709,-6.2207,-6.1706,-6.1204,-6.0702,-6.0201,-5.9699,-5.9197,-5.8696,-5.8194,-5.7692,-5.7191,-5.6689,-5.6187,-5.5686,-5.5184,-5.4682,-5.4181,-5.3679,-5.3177,-5.2676,-5.2174,-5.1672,-5.1171,-5.0669,-5.0167,-4.9666,-4.9164,-4.8662,-4.8161,-4.7659,-4.7157,-4.6656,-4.6154,-4.5652,-4.5151,-4.4649,-4.4147,-4.3645,-4.3144,-4.2642,-4.214,-4.1639,-4.1137,-4.0635,-4.0134,-3.9632,-3.913,-3.8629,-3.8127,-3.7625,-3.7124,-3.6622,-3.612,-3.5619,-3.5117,-3.4615,-3.4114,-3.3612,-3.311,-3.2609,-3.2107,-3.1605,-3.1104,-3.0602,-3.01,-2.9599,-2.9097,-2.8595,-2.8094,-2.7592,-2.709,-2.6589,-2.6087,-2.5585,-2.5084,-2.4582,-2.408,-2.3579,-2.3077,-2.2575,-2.2074,-2.1572,-2.107,-2.0569,-2.0067,-1.9565,-1.9064,-1.8562,-1.806,-1.7559,-1.7057,-1.6555,-1.6054,-1.5552,-1.505,-1.4548,-1.4047,-1.3545,-1.3043,-1.2542,-1.204,-1.1538,-1.1037,-1.0535,-1.0033,-0.95318,-0.90301,-0.85284,-0.80268,-0.75251,-0.70234,-0.65217,-0.60201,-0.55184,-0.50167,-0.45151,-0.40134,-0.35117,-0.301,-0.25084,-0.20067,-0.1505,-0.10033,-0.050167,0,0
};

//  Custom Trajectory Straight Line
//std::vector<double> x_waypoints =  {
//0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
//};

//std::vector<double> y_waypoints = {
//0,0.042607,0.085213,0.12782,0.17043,0.21303,0.25564,0.29825,0.34085,0.38346,0.42607,0.46867,0.51128,0.55388,0.59649,0.6391,0.6817,0.72431,0.76692,0.80952,0.85213,0.89474,0.93734,0.97995,1.0226,1.0652,1.1078,1.1504,1.193,1.2356,1.2782,1.3208,1.3634,1.406,1.4486,1.4912,1.5338,1.5764,1.619,1.6617,1.7043,1.7469,1.7895,1.8321,1.8747,1.9173,1.9599,2.0025,2.0451,2.0877,2.1303,2.1729,2.2155,2.2581,2.3008,2.3434,2.386,2.4286,2.4712,2.5138,2.5564,2.599,2.6416,2.6842,2.7268,2.7694,2.812,2.8546,2.8972,2.9398,2.9825,3.0251,3.0677,3.1103,3.1529,3.1955,3.2381,3.2807,3.3233,3.3659,3.4085,3.4511,3.4937,3.5363,3.5789,3.6216,3.6642,3.7068,3.7494,3.792,3.8346,3.8772,3.9198,3.9624,4.005,4.0476,4.0902,4.1328,4.1754,4.218,4.2607,4.3033,4.3459,4.3885,4.4311,4.4737,4.5163,4.5589,4.6015,4.6441,4.6867,4.7293,4.7719,4.8145,4.8571,4.8997,4.9424,4.985,5.0276,5.0702,5.1128,5.1554,5.198,5.2406,5.2832,5.3258,5.3684,5.411,5.4536,5.4962,5.5388,5.5815,5.6241,5.6667,5.7093,5.7519,5.7945,5.8371,5.8797,5.9223,5.9649,6.0075,6.0501,6.0927,6.1353,6.1779,6.2206,6.2632,6.3058,6.3484,6.391,6.4336,6.4762,6.5188,6.5614,6.604,6.6466,6.6892,6.7318,6.7744,6.817,6.8596,6.9023,6.9449,6.9875,7.0301,7.0727,7.1153,7.1579,7.2005,7.2431,7.2857,7.3283,7.3709,7.4135,7.4561,7.4987,7.5414,7.584,7.6266,7.6692,7.7118,7.7544,7.797,7.8396,7.8822,7.9248,7.9674,8.01,8.0526,8.0952,8.1378,8.1805,8.2231,8.2657,8.3083,8.3509,8.3935,8.4361,8.4787,8.5213,8.5639,8.6065,8.6491,8.6917,8.7343,8.7769,8.8195,8.8622,8.9048,8.9474,8.99,9.0326,9.0752,9.1178,9.1604,9.203,9.2456,9.2882,9.3308,9.3734,9.416,9.4586,9.5013,9.5439,9.5865,9.6291,9.6717,9.7143,9.7569,9.7995,9.8421,9.8847,9.9273,9.9699,10.013,10.055,10.098,10.14,10.183,10.226,10.268,10.311,10.353,10.396,10.439,10.481,10.524,10.566,10.609,10.652,10.694,10.737,10.779,10.822,10.865,10.907,10.95,10.992,11.035,11.078,11.12,11.163,11.206,11.248,11.291,11.333,11.376,11.419,11.461,11.504,11.546,11.589,11.632,11.674,11.717,11.759,11.802,11.845,11.887,11.93,11.972,12.015,12.058,12.1,12.143,12.185,12.228,12.271,12.313,12.356,12.398,12.441,12.484,12.526,12.569,12.612,12.654,12.697,12.739,12.782,12.825,12.867,12.91,12.952,12.995,13.038,13.08,13.123,13.165,13.208,13.251,13.293,13.336,13.378,13.421,13.464,13.506,13.549,13.591,13.634,13.677,13.719,13.762,13.805,13.847,13.89,13.932,13.975,14.018,14.06,14.103,14.145,14.188,14.231,14.273,14.316,14.358,14.401,14.444,14.486,14.529,14.571,14.614,14.657,14.699,14.742,14.784,14.827,14.87,14.912,14.955,14.997,15.04,15.083,15.125,15.168,15.211,15.253,15.296,15.338,15.381,15.424,15.466,15.509,15.551,15.594,15.637,15.679,15.722,15.764,15.807,15.85,15.892,15.935,15.977,16.02,16.063,16.105,16.148,16.19,16.233,16.276,16.318,16.361,16.404,16.446,16.489,16.531,16.574,16.617,16.659,16.702,16.744,16.787,16.83,16.872,16.915,16.957,17,17,16.957,16.915,16.872,16.83,16.787,16.744,16.702,16.659,16.617,16.574,16.531,16.489,16.446,16.404,16.361,16.318,16.276,16.233,16.19,16.148,16.105,16.063,16.02,15.977,15.935,15.892,15.85,15.807,15.764,15.722,15.679,15.637,15.594,15.551,15.509,15.466,15.424,15.381,15.338,15.296,15.253,15.211,15.168,15.125,15.083,15.04,14.997,14.955,14.912,14.87,14.827,14.784,14.742,14.699,14.657,14.614,14.571,14.529,14.486,14.444,14.401,14.358,14.316,14.273,14.231,14.188,14.145,14.103,14.06,14.018,13.975,13.932,13.89,13.847,13.805,13.762,13.719,13.677,13.634,13.591,13.549,13.506,13.464,13.421,13.378,13.336,13.293,13.251,13.208,13.165,13.123,13.08,13.038,12.995,12.952,12.91,12.867,12.825,12.782,12.739,12.697,12.654,12.612,12.569,12.526,12.484,12.441,12.398,12.356,12.313,12.271,12.228,12.185,12.143,12.1,12.058,12.015,11.972,11.93,11.887,11.845,11.802,11.759,11.717,11.674,11.632,11.589,11.546,11.504,11.461,11.419,11.376,11.333,11.291,11.248,11.206,11.163,11.12,11.078,11.035,10.992,10.95,10.907,10.865,10.822,10.779,10.737,10.694,10.652,10.609,10.566,10.524,10.481,10.439,10.396,10.353,10.311,10.268,10.226,10.183,10.14,10.098,10.055,10.013,9.9699,9.9273,9.8847,9.8421,9.7995,9.7569,9.7143,9.6717,9.6291,9.5865,9.5439,9.5013,9.4586,9.416,9.3734,9.3308,9.2882,9.2456,9.203,9.1604,9.1178,9.0752,9.0326,8.99,8.9474,8.9048,8.8622,8.8195,8.7769,8.7343,8.6917,8.6491,8.6065,8.5639,8.5213,8.4787,8.4361,8.3935,8.3509,8.3083,8.2657,8.2231,8.1805,8.1378,8.0952,8.0526,8.01,7.9674,7.9248,7.8822,7.8396,7.797,7.7544,7.7118,7.6692,7.6266,7.584,7.5414,7.4987,7.4561,7.4135,7.3709,7.3283,7.2857,7.2431,7.2005,7.1579,7.1153,7.0727,7.0301,6.9875,6.9449,6.9023,6.8596,6.817,6.7744,6.7318,6.6892,6.6466,6.604,6.5614,6.5188,6.4762,6.4336,6.391,6.3484,6.3058,6.2632,6.2206,6.1779,6.1353,6.0927,6.0501,6.0075,5.9649,5.9223,5.8797,5.8371,5.7945,5.7519,5.7093,5.6667,5.6241,5.5815,5.5388,5.4962,5.4536,5.411,5.3684,5.3258,5.2832,5.2406,5.198,5.1554,5.1128,5.0702,5.0276,4.985,4.9424,4.8997,4.8571,4.8145,4.7719,4.7293,4.6867,4.6441,4.6015,4.5589,4.5163,4.4737,4.4311,4.3885,4.3459,4.3033,4.2607,4.218,4.1754,4.1328,4.0902,4.0476,4.005,3.9624,3.9198,3.8772,3.8346,3.792,3.7494,3.7068,3.6642,3.6216,3.5789,3.5363,3.4937,3.4511,3.4085,3.3659,3.3233,3.2807,3.2381,3.1955,3.1529,3.1103,3.0677,3.0251,2.9825,2.9398,2.8972,2.8546,2.812,2.7694,2.7268,2.6842,2.6416,2.599,2.5564,2.5138,2.4712,2.4286,2.386,2.3434,2.3008,2.2581,2.2155,2.1729,2.1303,2.0877,2.0451,2.0025,1.9599,1.9173,1.8747,1.8321,1.7895,1.7469,1.7043,1.6617,1.619,1.5764,1.5338,1.4912,1.4486,1.406,1.3634,1.3208,1.2782,1.2356,1.193,1.1504,1.1078,1.0652,1.0226,0.97995,0.93734,0.89474,0.85213,0.80952,0.76692,0.72431,0.6817,0.6391,0.59649,0.55388,0.51128,0.46867,0.42607,0.38346,0.34085,0.29825,0.25564,0.21303,0.17043,0.12782,0.085213,0.042607,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
//};
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// RADIAL TEST
// std::vector<double> x_waypoints = {
//   1,1.2727,1.5455,1.8182,2.0909,2.3636,2.6364,2.9091,3.1818,3.4545,3.7273,4,4.2727,4.5455,4.8182,5.0909,5.3636,5.6364,5.9091,6.1818,6.4545,6.7273,7,7.2727,7.5455,7.8182,8.0909,8.3636,8.6364,8.9091,9.1818,9.4545,9.7273,10,10.273,10.545,10.818,11.091,11.364,11.636,11.909,12.182,12.455,12.727,13,13.273,13.545,13.818,14.091,14.364,14.636,14.909,15.182,15.455,15.727,16,16.273,16.545,16.818,17.091,17.364,17.636,17.909,18.182,18.455,18.727,19,19.273,19.545,19.818,20.091,20.364,20.636,20.909,21.182,21.455,21.727,22,22.273,22.545,22.818,23.091,23.364,23.636,23.909,24.182,24.455,24.727,25,25.273,25.545,25.818,26.091,26.364,26.636,26.909,27.182,27.455,27.727,28
// };
//
// std::vector<double> y_waypoints =  {
// 28,27.727,27.455,27.182,26.909,26.636,26.364,26.091,25.818,25.545,25.273,25,24.727,24.455,24.182,23.909,23.636,23.364,23.091,22.818,22.545,22.273,22,21.727,21.455,21.182,20.909,20.636,20.364,20.091,19.818,19.545,19.273,19,18.727,18.455,18.182,17.909,17.636,17.364,17.091,16.818,16.545,16.273,16,15.727,15.455,15.182,14.909,14.636,14.364,14.091,13.818,13.545,13.273,13,12.727,12.455,12.182,11.909,11.636,11.364,11.091,10.818,10.545,10.273,10,9.7273,9.4545,9.1818,8.9091,8.6364,8.3636,8.0909,7.8182,7.5455,7.2727,7,6.7273,6.4545,6.1818,5.9091,5.6364,5.3636,5.0909,4.8182,4.5455,4.2727,4,3.7273,3.4545,3.1818,2.9091,2.6364,2.3636,2.0909,1.8182,1.5455,1.2727,1
// };
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// BEARING TEST
// std::vector<double> x_waypoints = {
// -1.4696e-15,0.25382,0.50739,0.76045,1.0127,1.264,1.514,1.7625,2.0092,2.2539,2.4963,2.7362,2.9733,3.2074,3.4384,3.6658,3.8896,4.1094,4.3251,4.5365,4.7433,4.9453,5.1423,5.3342,5.5206,5.7016,5.8767,6.046,6.2092,6.3661,6.5166,6.6606,6.7978,6.9282,7.0516,7.168,7.2771,7.3788,7.4732,7.56,7.6392,7.7107,7.7745,7.8304,7.8785,7.9186,7.9507,7.9748,7.9909,7.999,7.999,7.9909,7.9748,7.9507,7.9186,7.8785,7.8304,7.7745,7.7107,7.6392,7.56,7.4732,7.3788,7.2771,7.168,7.0516,6.9282,6.7978,6.6606,6.5166,6.3661,6.2092,6.046,5.8767,5.7016,5.5206,5.3342,5.1423,4.9453,4.7433,4.5365,4.3251,4.1094,3.8896,3.6658,3.4384,3.2074,2.9733,2.7362,2.4963,2.2539,2.0092,1.7625,1.514,1.264,1.0127,0.76045,0.50739,0.25382,2.4493e-15,2.4493e-15,0.25382,0.50739,0.76045,1.0127,1.264,1.514,1.7625,2.0092,2.2539,2.4963,2.7362,2.9733,3.2074,3.4384,3.6658,3.8896,4.1094,4.3251,4.5365,4.7433,4.9453,5.1423,5.3342,5.5206,5.7016,5.8767,6.046,6.2092,6.3661,6.5166,6.6606,6.7978,6.9282,7.0516,7.168,7.2771,7.3788,7.4732,7.56,7.6392,7.7107,7.7745,7.8304,7.8785,7.9186,7.9507,7.9748,7.9909,7.999,7.999,7.9909,7.9748,7.9507,7.9186,7.8785,7.8304,7.7745,7.7107,7.6392,7.56,7.4732,7.3788,7.2771,7.168,7.0516,6.9282,6.7978,6.6606,6.5166,6.3661,6.2092,6.046,5.8767,5.7016,5.5206,5.3342,5.1423,4.9453,4.7433,4.5365,4.3251,4.1094,3.8896,3.6658,3.4384,3.2074,2.9733,2.7362,2.4963,2.2539,2.0092,1.7625,1.514,1.264,1.0127,0.76045,0.50739,0.25382,-1.4696e-15
// };
//
// std::vector<double> y_waypoints =  {
// -8,-7.996,-7.9839,-7.9638,-7.9356,-7.8995,-7.8554,-7.8034,-7.7436,-7.6759,-7.6006,-7.5175,-7.4269,-7.3289,-7.2234,-7.1107,-6.9908,-6.8639,-6.73,-6.5894,-6.4422,-6.2884,-6.1284,-5.9621,-5.7899,-5.6118,-5.4281,-5.2389,-5.0444,-4.8449,-4.6405,-4.4314,-4.2178,-4,-3.7782,-3.5525,-3.3233,-3.0908,-2.8551,-2.6165,-2.3754,-2.1318,-1.8861,-1.6385,-1.3892,-1.1385,-0.88671,-0.634,-0.38066,-0.12693,0.12693,0.38066,0.634,0.88671,1.1385,1.3892,1.6385,1.8861,2.1318,2.3754,2.6165,2.8551,3.0908,3.3233,3.5525,3.7782,4,4.2178,4.4314,4.6405,4.8449,5.0444,5.2389,5.4281,5.6118,5.7899,5.9621,6.1284,6.2884,6.4422,6.5894,6.73,6.8639,6.9908,7.1107,7.2234,7.3289,7.4269,7.5175,7.6006,7.6759,7.7436,7.8034,7.8554,7.8995,7.9356,7.9638,7.9839,7.996,8,8,7.996,7.9839,7.9638,7.9356,7.8995,7.8554,7.8034,7.7436,7.6759,7.6006,7.5175,7.4269,7.3289,7.2234,7.1107,6.9908,6.8639,6.73,6.5894,6.4422,6.2884,6.1284,5.9621,5.7899,5.6118,5.4281,5.2389,5.0444,4.8449,4.6405,4.4314,4.2178,4,3.7782,3.5525,3.3233,3.0908,2.8551,2.6165,2.3754,2.1318,1.8861,1.6385,1.3892,1.1385,0.88671,0.634,0.38066,0.12693,-0.12693,-0.38066,-0.634,-0.88671,-1.1385,-1.3892,-1.6385,-1.8861,-2.1318,-2.3754,-2.6165,-2.8551,-3.0908,-3.3233,-3.5525,-3.7782,-4,-4.2178,-4.4314,-4.6405,-4.8449,-5.0444,-5.2389,-5.4281,-5.6118,-5.7899,-5.9621,-6.1284,-6.2884,-6.4422,-6.5894,-6.73,-6.8639,-6.9908,-7.1107,-7.2234,-7.3289,-7.4269,-7.5175,-7.6006,-7.6759,-7.7436,-7.8034,-7.8554,-7.8995,-7.9356,-7.9638,-7.9839,-7.996,-8
// };
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// AZIMUTH TEST
// std::vector<double> x_waypoints = {
// -1.4696e-15,0.12693,0.25382,0.38066,0.50739,0.634,0.76045,0.88671,1.0127,1.1385,1.264,1.3892,1.514,1.6385,1.7625,1.8861,2.0092,2.1318,2.2539,2.3754,2.4963,2.6165,2.7362,2.8551,2.9733,3.0908,3.2074,3.3233,3.4384,3.5525,3.6658,3.7782,3.8896,4,4.1094,4.2178,4.3251,4.4314,4.5365,4.6405,4.7433,4.8449,4.9453,5.0444,5.1423,5.2389,5.3342,5.4281,5.5206,5.6118,5.7016,5.7899,5.8767,5.9621,6.046,6.1284,6.2092,6.2884,6.3661,6.4422,6.5166,6.5894,6.6606,6.73,6.7978,6.8639,6.9282,6.9908,7.0516,7.1107,7.168,7.2234,7.2771,7.3289,7.3788,7.4269,7.4732,7.5175,7.56,7.6006,7.6392,7.6759,7.7107,7.7436,7.7745,7.8034,7.8304,7.8554,7.8785,7.8995,7.9186,7.9356,7.9507,7.9638,7.9748,7.9839,7.9909,7.996,7.999,8,8,7.999,7.996,7.9909,7.9839,7.9748,7.9638,7.9507,7.9356,7.9186,7.8995,7.8785,7.8554,7.8304,7.8034,7.7745,7.7436,7.7107,7.6759,7.6392,7.6006,7.56,7.5175,7.4732,7.4269,7.3788,7.3289,7.2771,7.2234,7.168,7.1107,7.0516,6.9908,6.9282,6.8639,6.7978,6.73,6.6606,6.5894,6.5166,6.4422,6.3661,6.2884,6.2092,6.1284,6.046,5.9621,5.8767,5.7899,5.7016,5.6118,5.5206,5.4281,5.3342,5.2389,5.1423,5.0444,4.9453,4.8449,4.7433,4.6405,4.5365,4.4314,4.3251,4.2178,4.1094,4,3.8896,3.7782,3.6658,3.5525,3.4384,3.3233,3.2074,3.0908,2.9733,2.8551,2.7362,2.6165,2.4963,2.3754,2.2539,2.1318,2.0092,1.8861,1.7625,1.6385,1.514,1.3892,1.264,1.1385,1.0127,0.88671,0.76045,0.634,0.50739,0.38066,0.25382,0.12693,-1.4696e-15
// };
//
// std::vector<double> y_waypoints =  {
// 0,0.001007,0.0040277,0.0090613,0.016107,0.025162,0.036225,0.049292,0.064361,0.081428,0.10049,0.12154,0.14457,0.16958,0.19656,0.22551,0.25641,0.28926,0.32406,0.36078,0.39943,0.43999,0.48246,0.52682,0.57306,0.62117,0.67113,0.72294,0.77659,0.83205,0.88932,0.94837,1.0092,1.0718,1.1361,1.2022,1.27,1.3394,1.4106,1.4834,1.5578,1.6339,1.7116,1.7908,1.8716,1.954,2.0379,2.1233,2.2101,2.2984,2.3882,2.4794,2.5719,2.6658,2.7611,2.8577,2.9556,3.0547,3.1551,3.2567,3.3595,3.4635,3.5686,3.6749,3.7822,3.8906,4,4.1104,4.2218,4.3342,4.4475,4.5616,4.6767,4.7926,4.9092,5.0267,5.1449,5.2638,5.3835,5.5037,5.6246,5.7461,5.8682,5.9908,6.1139,6.2375,6.3615,6.486,6.6108,6.736,6.8615,6.9873,7.1133,7.2396,7.366,7.4926,7.6193,7.7462,7.8731,8,8,7.8731,7.7462,7.6193,7.4926,7.366,7.2396,7.1133,6.9873,6.8615,6.736,6.6108,6.486,6.3615,6.2375,6.1139,5.9908,5.8682,5.7461,5.6246,5.5037,5.3835,5.2638,5.1449,5.0267,4.9092,4.7926,4.6767,4.5616,4.4475,4.3342,4.2218,4.1104,4,3.8906,3.7822,3.6749,3.5686,3.4635,3.3595,3.2567,3.1551,3.0547,2.9556,2.8577,2.7611,2.6658,2.5719,2.4794,2.3882,2.2984,2.2101,2.1233,2.0379,1.954,1.8716,1.7908,1.7116,1.6339,1.5578,1.4834,1.4106,1.3394,1.27,1.2022,1.1361,1.0718,1.0092,0.94837,0.88932,0.83205,0.77659,0.72294,0.67113,0.62117,0.57306,0.52682,0.48246,0.43999,0.39943,0.36078,0.32406,0.28926,0.25641,0.22551,0.19656,0.16958,0.14457,0.12154,0.10049,0.081428,0.064361,0.049292,0.036225,0.025162,0.016107,0.0090613,0.0040277,0.001007,0
// };


 int counter = 0;
}
#endif
